/*************************************************************************************************************************
									File Details
**************************************************************************************************************************
	File name 							: SystemInit.C
	
	Purpose								: This file has driver functions to initialize processor specific modules such as
									      PLL, SRU, DDR2/SDRAM, AMI controller  etc. These functions are called single time 
									      at the begining of the code.
	
	Author								: Mitesh Moonat & Harshit Gaharwar
	
	VDSP version used for testing		: VisualDSP++ 5.0, Update 10
	
	Hardware used						: ADSP-21469, ADSP-21479, ADSP-21489, ADSP-21375, ADSP-21369, ADSP-21364,
										  ADSP-21262  EZ-Kits
	
	Connection details 					: See "main.C"
	
	Guidelines for change				: 1) Following changes may be required for different processors or FLASH device
											
										  - The CCLK, SDCLK/DDR2CLK, SRU settings are currently set similar to the 
										  	EZ-Kit examples.They can/should be changed if required for custom board. 
										  	One should change the variables g_fPCLK accordingly.
										  	
										  - No changes required to this file if the code has to run on any of the EZ-Kits
**************************************************************************************************************************/

/*************************************************************************************************************************
									Include Files
*************************************************************************************************************************/
#include "SystemInit.h"



/*************************************************************************************************************************
									Global Variables
**************************************************************************************************************************/
float g_fPCLK = 200000000.0;  //PCLK = 200 MHz default for ADSP-2488/ADSP-21489




#if defined(__ADSP21367__) || defined(__ADSP21368__) || \
    	defined(__ADSP21369__) || defined(__2137x__) || defined(__214xx__)
    

/**************************************************************************************************************************
	Function name	:   Init_SRU
    Description		:   This function initializes the SRU
	Arguments		:	None
	Return value	:   None
**************************************************************************************************************************/
void Init_SRU(void)
{

    SRU(SPI_MOSI_O,DPI_PB01_I);      //Connect MOSI to DPI PB1.
    SRU(DPI_PB02_O, SPI_MISO_I);     //Connect DPI PB2 to MISO.
    SRU(SPI_CLK_O, DPI_PB03_I);      //Connect SPI CLK to DPI PB3.
   // SRU(SPI_FLG0_O, DPI_PB05_I);     //Connect SPI FLAG3 to DPI PB5.
   
    SRU(HIGH, DPI_PB05_I);     //Drive the CS high initially
    
    SRU(HIGH, DPI_PBEN01_I);
    SRU(LOW, DPI_PBEN02_I);
    SRU(SPI_CLK_PBEN_O, DPI_PBEN03_I);
    SRU(HIGH, DPI_PBEN05_I);	
}

#endif

/**************************************************************************************************************************
	Function name	:   Setup_System_For_FLASH
    Description		:   This function initializes the system (PLL, SRU, SPI baud rate etc) for the FLASH
	Arguments		:	Parameter			 |	Description					| Valid values
						fSPICLK				 |  SPICLK speed 			    | Float
	Return value	:   None
**************************************************************************************************************************/
void Setup_System_For_FLASH(float fSPICLK)
{
	//SRU Initilzation required only for processors 21367/8/9 onwards
	#if defined(__ADSP21367__) || defined(__ADSP21368__) || \
    	defined(__ADSP21369__) || defined(__2137x__) || defined(__214xx__)
    
		Init_SRU();
		
	#else	
		//Set FLAG0 as output
		asm("bit set FLAGS 0x3;");
		
	#endif
	
	SPI_Set_Baud_Rate(fSPICLK, g_fPCLK);
}

